"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const child_process_1 = require("child_process");
const fs = __importStar(require("fs"));
const Path = __importStar(require("path"));
const qualitygate_1 = require("../../lib/qualitygate");
const requirements_1 = require("../../lib/requirements");
// Initialize Messages with the current plugin directory
core_1.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = core_1.Messages.loadMessages('sfdx-codescan-plugin', 'codescan');
class Run extends command_1.SfdxCommand {
    static description = messages.getMessage('runCommandDescription');
    static examples = [
        `$ sfdx codescan:run --token <token> --projectkey my-project-key --organization my-org-key
  `,
        `$ sfdx codescan:run --token <token> --projectkey my-project-key --organization my-org-key -Dsonar.verbose=true
    -D can be used for passing any sonar-scanner definition
    -J will be passed as a jvm arg
  `,
        `$ sfdx codescan:run ... -X
    Verbose output`
    ];
    static args = [];
    static varargs = true;
    static flagsConfig = {
        // flag with a value (-n, --name=VALUE)
        server: command_1.flags.string({ char: 's', description: messages.getMessage('serverFlagDescription') }),
        organization: command_1.flags.string({ char: 'o', description: messages.getMessage('organizationFlagDescription') }),
        projectkey: command_1.flags.string({ char: 'k', description: messages.getMessage('projectKeyFlagDescription') }),
        token: command_1.flags.string({ char: 't', description: messages.getMessage('tokenFlagDescription') }),
        username: command_1.flags.string({ char: 'u', description: messages.getMessage('usernameFlagDescription') }),
        password: command_1.flags.string({ char: 'p', description: messages.getMessage('passwordFlagDescription') }),
        noqualitygate: command_1.flags.boolean({ description: messages.getMessage('noqualitygateFlagDescription') }),
        javahome: command_1.flags.string({ description: messages.getMessage('javahomeFlagDescription') }),
        nofail: command_1.flags.boolean({ description: messages.getMessage('nofailFlagDescription') }),
        qgtimeout: command_1.flags.integer({ description: messages.getMessage('qgtimeoutFlagDescription') })
    };
    static SONAR_SCANNER_VERSION = '5.0.1.3007';
    varargValues = [];
    codescanPath = null;
    async run() {
        const varargs = this.varargValues.slice();
        const command = await this.resolveJava(this.flags.javahome);
        let args = [];
        if (this.contains(varargs, '-Dsonar.host.url')) {
            // skip
        }
        else if (!this.flags.server) {
            this.flags.server = 'https://app.codescan.io';
        }
        else if (!this.flags.server.match(/https?:\/\/[A-Za-z0-9\-_]+(\..+|:[0-9]+\/*.*|\/.*)/)) {
            throw new core_1.SfError(messages.getMessage('errorInvalidServerUrl'));
        }
        if (this.flags.server && this.flags.server.endsWith('codescan.io') && !this.flags.organization) {
            throw new core_1.SfError(messages.getMessage('errorNoOrganization'));
        }
        // put -J vm args at front...
        for (const x in varargs) {
            if (varargs[x].startsWith('-J')) {
                args.push(varargs[x].substring(2));
            }
        }
        // jar
        args.push('-jar');
        args.push(await this.resolveSonarScanner());
        // server
        if (this.flags.server) {
            if (this.contains(varargs, '-Dsonar.host.url')) {
                throw new core_1.SfError(messages.getMessage('errorDuplicateValues', ['server']));
            }
            args.push('-Dsonar.host.url=' + this.flags.server);
        }
        if (this.flags.organization) {
            if (this.contains(varargs, '-Dsonar.organization')) {
                throw new core_1.SfError(messages.getMessage('errorDuplicateValues', ['organization']));
            }
            args.push('-Dsonar.organization=' + this.flags.organization);
        }
        // authentication
        if (this.contains(varargs, '-Dsonar.login') || this.contains(varargs, '-Dsonar.password')) {
            if (this.flags.token || this.flags.username && this.flags.password) {
                throw new core_1.SfError(messages.getMessage('errorDuplicateValues', ['token/username/password']));
            }
        }
        else if (this.flags.token) {
            if (this.flags.username || this.flags.password) {
                throw new core_1.SfError(messages.getMessage('errorTokenAndUserPass'));
            }
            args.push('-Dsonar.login=' + this.flags.token);
        }
        else if (this.flags.username && this.flags.password) {
            args.push('-Dsonar.login=' + this.flags.username);
            args.push('-Dsonar.password=' + this.flags.password);
        }
        else if (this.flags.username || this.flags.password) {
            throw new core_1.SfError(messages.getMessage('errorOnlyUserOrPass'));
        }
        else {
            this.ux.log('No --token specified. This will probably fail');
        }
        if (this.flags.projectkey) {
            if (this.contains(varargs, '-Dsonar.projectKey')) {
                throw new core_1.SfError(messages.getMessage('errorDuplicateValues', ['projectkey']));
            }
            args.push('-Dsonar.projectKey=' + this.flags.projectkey);
        }
        // setup working dir
        let sonarWorkingDir = this.contains(varargs, '-Dsonar.working.directory=');
        if (sonarWorkingDir) {
            sonarWorkingDir = sonarWorkingDir.substring('-Dsonar.working.directory='.length);
        }
        else {
            sonarWorkingDir = Path.join(this.codescanPath, 'sonarworker');
            args.push('-Dsonar.working.directory=' + sonarWorkingDir);
        }
        // add rest of varargs
        for (const x in varargs) {
            if (!varargs[x].startsWith('-J')) {
                args.push(varargs[x]);
            }
        }
        // spawn scanner
        this.ux.log(`\nRunning command \n[${command}] \nwith arguments \n${args.join('\n')}\n`);
        const cmd = (0, child_process_1.spawn)(command, args);
        cmd.stdout.pipe(process.stdout);
        cmd.stderr.pipe(process.stderr);
        const nofail = !!this.flags.nofail;
        const noqualitygate = !!this.flags.noqualitygate;
        const getQualityGate = this.getQualityGate;
        const _this = this;
        return new Promise((resolve, reject) => {
            cmd.addListener('exit', code => {
                if (code === 0 && !noqualitygate) {
                    _this.ux.startSpinner('Waiting for SonarQube Background Task...');
                    return getQualityGate(_this, sonarWorkingDir)
                        .catch(error => {
                        _this.ux.stopSpinner();
                        if (_this.flags.nofail) {
                            return resolve({ code: 0, error });
                        }
                        else {
                            return reject(error);
                        }
                    })
                        .then(ret => {
                        if (!ret) {
                            return;
                        }
                        _this.ux.stopSpinner();
                        return _this.printQualityGate(ret, nofail, resolve, reject);
                    });
                }
                else if (code === 0 || nofail) {
                    resolve({ code });
                }
                else {
                    reject({ code });
                }
            });
        });
    }
    parseVarargs(args) {
        this.varargValues = args;
        return {};
    }
    printQualityGate(qualitygate, nofail, resolve, reject) {
        const chalk = require('chalk');
        let code = 0;
        if (qualitygate['status'] === 'OK') {
            this.ux.log(chalk.green('Quality Gate passed'));
        }
        else {
            this.ux.error(chalk.red('Quality Gate failed'));
            this.ux.error(chalk.red('See more details on the dashboard URL above'));
            code = 1;
            if (!nofail)
                return reject('Quality Gate failed');
        }
        return resolve({ code, qualitygate });
    }
    getQualityGate(_this, sonarWorkingDir) {
        const qgtimeout = _this.flags.qgtimeout ? parseInt(_this.flags.qgtimeout, 10) : 300;
        const end = new Date().getTime() + (qgtimeout * 1000);
        const auth = _this.flags.token ? _this.flags.token : (_this.flags.username && _this.flags.password ? { user: _this.flags.username, pass: _this.flags.password } : {});
        return new Promise((resolve, reject) => {
            (0, qualitygate_1.pollQualityGate)(auth, end, sonarWorkingDir, 2000, resolve, reject);
        });
    }
    resolveJava(javaHome) {
        return (0, requirements_1.resolveRequirements)(javaHome)
            .catch(error => {
            throw error;
        })
            .then(requirements => {
            return requirements.javaHome + '/bin/java';
        });
    }
    resolveSonarScanner() {
        // create parent dirs
        if (!fs.existsSync(Path.normalize('.sfdx/'))) {
            fs.mkdirSync(Path.normalize('.sfdx/'));
        }
        this.codescanPath = this.codescanPath = Path.normalize('.sfdx/codescan');
        if (!fs.existsSync(this.codescanPath)) {
            fs.mkdirSync(this.codescanPath);
        }
        // copy sonar scanner from assets folder
        const copydir = require('copy-dir');
        const sonarScannerSourcePath = Path.normalize(__dirname + '../../../../assets');
        this.ux.log(`Copying sonar scanner binaries from [${sonarScannerSourcePath}]...`);
        copydir.sync(sonarScannerSourcePath, '.sfdx/codescan', {
            utimes: true,
            mode: true,
            cover: true // cover file when exists, default is true
        });
        return Path.join(this.codescanPath, 'sonar-scanner-' + Run.SONAR_SCANNER_VERSION + '/lib/sonar-scanner-cli-' + Run.SONAR_SCANNER_VERSION + '.jar');
    }
    contains(arr, prefix) {
        for (const x in arr) {
            if (arr[x].startsWith(prefix)) {
                return arr[x];
            }
        }
        return false;
    }
}
exports.default = Run;
//# sourceMappingURL=run.js.map