"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.pollQualityGate = void 0;
const fs = __importStar(require("fs"));
const Path = __importStar(require("path"));
const axios_1 = __importDefault(require("axios"));
function poll(pollFn, interval = 100) {
    let intervalHandle = null;
    return {
        until(conditionFn) {
            return new Promise((resolve, reject) => {
                intervalHandle = setInterval(() => {
                    pollFn().then(data => {
                        let passesCondition = false;
                        try {
                            passesCondition = conditionFn(data);
                        }
                        catch (e) {
                            reject(e);
                        }
                        if (passesCondition) {
                            clearInterval(intervalHandle);
                            resolve(data);
                        }
                    }).catch(err => {
                        clearInterval(intervalHandle);
                        reject(err);
                    });
                }, interval);
            });
        }
    };
}
function getCeTaskUrl(sonarWorkingDir) {
    const lines = fs.readFileSync(Path.join(sonarWorkingDir, 'report-task.txt'), 'utf-8').split('\n');
    for (const i in lines) {
        if (lines[i].startsWith('ceTaskUrl=')) {
            return lines[i].substring('ceTaskUrl='.length);
        }
    }
    return null;
}
function getQualityGateUrl(sonarWorkingDir, ceTask) {
    let analysisId = null;
    if (ceTask['analysisId']) {
        analysisId = ceTask['analysisId'];
    }
    else {
        analysisId = ceTask['id'];
    }
    const lines = fs.readFileSync(Path.join(sonarWorkingDir, 'report-task.txt'), 'utf-8').split('\n');
    let projectKey = null;
    let serverUrl = null;
    for (const i in lines) {
        if (lines[i].startsWith('serverUrl=')) {
            serverUrl = lines[i].substring('serverUrl='.length);
        }
        else if (lines[i].startsWith('projectKey=')) {
            projectKey = lines[i].substring('projectKey='.length);
        }
    }
    if (serverUrl && projectKey) {
        return serverUrl + '/api/qualitygates/project_status?analysisId=' + analysisId;
    }
    return null;
}
function pollQualityGate(auth, end, sonarWorkingDir, interval, resolve, reject) {
    const url = getCeTaskUrl(sonarWorkingDir);
    if (!url) {
        reject('ceTaskUrl not found');
        return;
    }
    poll(() => {
        return new Promise((_resolve, _reject) => {
            axios_1.default.get(url, { headers: { Authorization: `Basic ${btoa(auth + ':')}` } })
                .then(response => {
                const data = response.data;
                if (data.errors) {
                    _reject(data.errors[0].msg);
                }
                _resolve(data.task);
            })
                .catch(error => _reject(error));
        });
    }, interval)
        .until(data => {
        return (data.status !== 'IN_PROGRESS' && data.status !== 'PENDING') || new Date().getTime() >= end;
    })
        .then(ceTask => {
        if (ceTask['status'] === 'IN_PROGRESS' || ceTask['status'] === 'PENDING') {
            reject('Quality Gate Timeout');
        }
        else {
            const qgurl = getQualityGateUrl(sonarWorkingDir, ceTask);
            if (!qgurl) {
                reject('qualityGate url not found');
            }
            else {
                // fetch quality gate...
                axios_1.default.get(qgurl, { headers: { Authorization: `Basic ${btoa(auth + ':')}` } })
                    .then(response => {
                    const data = response.data;
                    if (data.errors) {
                        reject(data.errors[0].msg);
                    }
                    resolve(data.projectStatus);
                })
                    .catch(error => reject(error));
            }
        }
    })
        .catch(reject);
}
exports.pollQualityGate = pollQualityGate;
//# sourceMappingURL=qualitygate.js.map